@@ -58,7 +58,11 @@
 #elif defined(_MSC_VER)
 
 #define MPT_COMPILER_MSVC                            1
-#if (_MSC_VER >= 1922)
+#if (_MSC_VER >= 1924)
+#define MPT_COMPILER_MSVC_VERSION                    MPT_COMPILER_MAKE_VERSION2(2019,4)
+#elif (_MSC_VER >= 1923)
+#define MPT_COMPILER_MSVC_VERSION                    MPT_COMPILER_MAKE_VERSION2(2019,3)
+#elif (_MSC_VER >= 1922)
 #define MPT_COMPILER_MSVC_VERSION                    MPT_COMPILER_MAKE_VERSION2(2019,2)
 #elif (_MSC_VER >= 1921)
 #define MPT_COMPILER_MSVC_VERSION                    MPT_COMPILER_MAKE_VERSION2(2019,1)
@@ -232,6 +236,8 @@
 	//#elif TARGET_OS_MAC
 	//#else
 	//#endif
+#elif defined(__HAIKU__)
+	#define MPT_OS_HAIKU 1
 #elif defined(__ANDROID__) || defined(ANDROID)
 	#define MPT_OS_ANDROID 1
 #elif defined(__linux__)
@@ -265,6 +271,9 @@
 #ifndef MPT_OS_MACOSX_OR_IOS
 #define MPT_OS_MACOSX_OR_IOS 0
 #endif
+#ifndef MPT_OS_HAIKU
+#define MPT_OS_HAIKU 0
+#endif
 #ifndef MPT_OS_ANDROID
 #define MPT_OS_ANDROID 0
 #endif
@@ -311,9 +320,3 @@
 #define MPT_COMPILER_QUIRK_MSVC_STRINGSTREAM
 #endif
 
-#if MPT_CXX_AT_LEAST(17)
-#if MPT_COMPILER_MSVC || MPT_GCC_BEFORE(8,1,0) || MPT_CLANG_BEFORE(5,0,0) || (MPT_COMPILER_GCC && defined(__GLIBCXX__) && (defined(__MINGW32__) || defined(__MINGW64__))) || (MPT_COMPILER_CLANG && defined(__GLIBCXX__)) || (MPT_COMPILER_CLANG && MPT_OS_MACOSX_OR_IOS) || MPT_OS_OPENBSD || MPT_OS_EMSCRIPTEN || (defined(__clang__) && defined(_MSC_VER))
-#define MPT_COMPILER_QUIRK_NO_ALIGNEDALLOC
-#endif
-#endif
-
