@@ -221,6 +221,14 @@ void PathString::SplitPath(PathString *drive, PathString *dir, PathString *fname
 	}
 	mpt::RawPathString::size_type last_dot = p.find_last_of(PL_("."));
 	if(last_dot == mpt::RawPathString::npos)
+	{
+		if(fname) *fname = mpt::PathString::FromNative(p);
+		if(ext) *ext = mpt::PathString();
+	} else if(last_dot == 0)
+	{
+		if(fname) *fname = mpt::PathString::FromNative(p);
+		if(ext) *ext = mpt::PathString();
+	} else if(p == PL_(".") || p == PL_(".."))
 	{
 		if(fname) *fname = mpt::PathString::FromNative(p);
 		if(ext) *ext = mpt::PathString();
