@@ -41,34 +41,33 @@ Metadata and other state is not provided or updated.
 #include <string>
 #include <vector>
 
-#include <climits>
 #include <cmath>
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
 
+#define MODPLUG_BUILD
+#ifdef _MSC_VER
+/* libmodplug C++ header is broken for MSVC DLL builds */
+#define MODPLUG_STATIC
+#endif /* _MSC_VER */
 #ifdef _MSC_VER
-/* msvc errors when seeing dllexport declarations after prototypes have been declared in modplug.h */
 #define LIBOPENMPT_MODPLUG_API
 #else /* !_MSC_VER */
 #define LIBOPENMPT_MODPLUG_API LIBOPENMPT_API_HELPER_EXPORT
 #endif /* _MSC_VER */
+class LIBOPENMPT_MODPLUG_API CSoundFile;
+#include "libmodplug/stdafx.h"
+#include "libmodplug/sndfile.h"
 
 namespace {
-
 template <class T>
 void Clear( T & x )
 {
 	std::memset( &x, 0, sizeof(T) );
 }
-
 }
 
-class LIBOPENMPT_MODPLUG_API CSoundFile;
-
-#include "libmodplug/stdafx.h"
-#include "libmodplug/sndfile.h"
-
 //#define mpcpplog() fprintf(stderr, "%s %i\n", __func__, __LINE__)
 #define mpcpplog() do{}while(0)
 
@@ -189,7 +188,7 @@ BOOL CSoundFile::Create( LPCBYTE lpStream, DWORD dwMemLength ) {
 	try {
 		openmpt::module * m = new openmpt::module( lpStream, dwMemLength );
 		set_self( this, m );
-		std::strncpy( m_szNames[0], mod->get_metadata("title").c_str(), sizeof( m_szNames[0] ) );
+		std::strncpy( m_szNames[0], mod->get_metadata("title").c_str(), sizeof( m_szNames[0] ) - 1 );
 		m_szNames[0][ sizeof( m_szNames[0] ) - 1 ] = '\0';
 		std::string type = mod->get_metadata("type");
 		m_nType = MOD_TYPE_NONE;
@@ -305,9 +304,6 @@ void CSoundFile::SetCurrentOrder( UINT nOrder ) {
 
 UINT CSoundFile::GetSampleName( UINT nSample, LPSTR s ) const {
 	mpcpplog();
-	if ( !s ) {
-		return 0;
-	}
 	char buf[32];
 	std::memset( buf, 0, 32 );
 	if ( mod ) {
@@ -316,15 +312,14 @@ UINT CSoundFile::GetSampleName( UINT nSample, LPSTR s ) const {
 			std::strncpy( buf, names[ nSample - 1 ].c_str(), 31 );
 		}
 	}
-	std::memcpy( s, buf, 32 );
+	if ( s ) {
+		std::strncpy( s, buf, 32 );
+	}
 	return static_cast<UINT>( std::strlen( buf ) );
 }
 
 UINT CSoundFile::GetInstrumentName( UINT nInstr, LPSTR s ) const {
 	mpcpplog();
-	if ( !s ) {
-		return 0;
-	}
 	char buf[32];
 	std::memset( buf, 0, 32 );
 	if ( mod ) {
@@ -333,7 +328,9 @@ UINT CSoundFile::GetInstrumentName( UINT nInstr, LPSTR s ) const {
 			std::strncpy( buf, names[ nInstr - 1 ].c_str(), 31 );
 		}
 	}
-	std::memcpy( s, buf, 32 );
+	if ( s ) {
+		std::strncpy( s, buf, 32 );
+	}
 	return static_cast<UINT>( std::strlen( buf ) );
 }
 
@@ -678,7 +675,7 @@ static int get_filter_length() {
 		return 8;
 	} else if ( ( CSoundFile::gdwSoundSetup & SNDMIX_HQRESAMPLER ) == SNDMIX_HQRESAMPLER ) {
 		return 4;
-	} else if ( ( CSoundFile::gdwSoundSetup & SNDMIX_HQRESAMPLER ) == SNDMIX_NORESAMPLING ) {
+	} else if ( ( CSoundFile::gdwSoundSetup & SNDMIX_NORESAMPLING ) == SNDMIX_NORESAMPLING ) {
 		return 1;
 	} else {
 		return 2;
