@@ -12,7 +12,6 @@
 #include "stdafx.h"
 #include "Loaders.h"
 #include "../common/version.h"
-#include "../common/misc_util.h"
 #include "XMTools.h"
 #ifndef MODPLUG_NO_FILESAVE
 #include "../common/mptFileIO.h"
@@ -43,7 +42,7 @@ static std::vector<SAMPLEINDEX> AllocateXMSamples(CSoundFile &sndFile, SAMPLEIND
 			// If too many sample slots are needed, try to fill some empty slots first.
 			for(SAMPLEINDEX j = 1; j <= sndFile.GetNumSamples(); j++)
 			{
-				if(sndFile.GetSample(j).pSample != nullptr)
+				if(sndFile.GetSample(j).HasSampleData())
 				{
 					continue;
 				}
@@ -321,6 +320,7 @@ bool CSoundFile::ReadXM(FileReader &file, ModLoadingFlags loadFlags)
 	m_nMixLevels = mixLevelsCompatible;
 
 	FlagSet<TrackerVersions> madeWith(verUnknown);
+	mpt::ustring madeWithTracker;
 
 	if(!memcmp(fileHeader.trackerName, "FastTracker ", 12))
 	{
@@ -341,14 +341,14 @@ bool CSoundFile::ReadXM(FileReader &file, ModLoadingFlags loadFlags)
 		{
 			// ???
 			madeWith.set(verConfirmed);
-			m_madeWithTracker = MPT_USTRING("FastTracker Clone");
+			madeWithTracker = U_("FastTracker Clone");
 		}
 	} else
 	{
 		// Something else!
 		madeWith = verUnknown | verConfirmed;
 
-		mpt::String::Read<mpt::String::spacePadded>(m_madeWithTracker, mpt::CharsetCP437, fileHeader.trackerName);
+		mpt::String::Read<mpt::String::spacePadded>(madeWithTracker, mpt::CharsetCP437, fileHeader.trackerName);
 
 		if(!memcmp(fileHeader.trackerName, "OpenMPT ", 8))
 		{
@@ -438,12 +438,12 @@ bool CSoundFile::ReadXM(FileReader &file, ModLoadingFlags loadFlags)
 			{
 				// ModPlug Tracker Alpha
 				m_dwLastSavedWithVersion = MAKE_VERSION_NUMERIC(1, 00, 00, A5);
-				m_madeWithTracker = MPT_USTRING("ModPlug Tracker 1.0 alpha");
+				madeWithTracker = U_("ModPlug Tracker 1.0 alpha");
 			} else if(instrHeader.size == 263)
 			{
 				// ModPlug Tracker Beta (Beta 1 still behaves like Alpha, but Beta 3.3 does it this way)
 				m_dwLastSavedWithVersion = MAKE_VERSION_NUMERIC(1, 00, 00, B3);
-				m_madeWithTracker = MPT_USTRING("ModPlug Tracker 1.0 beta");
+				madeWithTracker = U_("ModPlug Tracker 1.0 beta");
 			} else
 			{
 				// WTF?
@@ -640,11 +640,11 @@ bool CSoundFile::ReadXM(FileReader &file, ModLoadingFlags loadFlags)
 		if(madeWith[verModPlug1_09])
 		{
 			m_dwLastSavedWithVersion = MAKE_VERSION_NUMERIC(1, 09, 00, 00);
-			m_madeWithTracker = MPT_USTRING("ModPlug Tracker 1.09");
+			madeWithTracker = U_("ModPlug Tracker 1.09");
 		} else if(madeWith[verNewModPlug])
 		{
 			m_dwLastSavedWithVersion = MAKE_VERSION_NUMERIC(1, 16, 00, 00);
-			m_madeWithTracker = MPT_USTRING("ModPlug Tracker 1.10 - 1.16");
+			madeWithTracker = U_("ModPlug Tracker 1.10 - 1.16");
 		}
 	}
 
@@ -652,7 +652,7 @@ bool CSoundFile::ReadXM(FileReader &file, ModLoadingFlags loadFlags)
 	{
 		// Hey, I know this tracker!
 		std::string mptVersion(fileHeader.trackerName + 8, 12);
-		m_dwLastSavedWithVersion = MptVersion::ToNum(mptVersion);
+		m_dwLastSavedWithVersion = Version::Parse(mpt::ToUnicode(mpt::CharsetASCII, mptVersion));
 		madeWith = verOpenMPT | verConfirmed;
 
 		if(m_dwLastSavedWithVersion < MAKE_VERSION_NUMERIC(1, 22, 07, 19))
@@ -661,7 +661,7 @@ bool CSoundFile::ReadXM(FileReader &file, ModLoadingFlags loadFlags)
 			m_nMixLevels = mixLevelsCompatibleFT2;
 	}
 
-	if(m_dwLastSavedWithVersion != 0 && !madeWith[verOpenMPT])
+	if(m_dwLastSavedWithVersion && !madeWith[verOpenMPT])
 	{
 		m_nMixLevels = mixLevelsOriginal;
 		m_playBehaviour.reset();
@@ -689,32 +689,29 @@ bool CSoundFile::ReadXM(FileReader &file, ModLoadingFlags loadFlags)
 		}
 	}
 
-	if(m_madeWithTracker.empty())
+	if(madeWithTracker.empty())
 	{
 		if(madeWith[verDigiTrakker] && sampleReserved == 0 && (instrType ? instrType : -1) == -1)
 		{
-			m_madeWithTracker = MPT_USTRING("DigiTrakker");
+			madeWithTracker = U_("DigiTrakker");
 		} else if(madeWith[verFT2Generic])
 		{
-			m_madeWithTracker = MPT_USTRING("FastTracker 2 or compatible");
+			madeWithTracker = U_("FastTracker 2 or compatible");
 		} else
 		{
-			m_madeWithTracker = MPT_USTRING("Unknown");
+			madeWithTracker = U_("Unknown");
 		}
 	}
 
-	// Leave if no extra instrument settings are available (end of file reached)
-	if(file.NoBytesLeft()) return true;
-
-	bool interpretOpenMPTMade = false; // specific for OpenMPT 1.17+ (bMadeWithModPlug is also for MPT 1.16)
+	bool isOpenMPTMade = false; // specific for OpenMPT 1.17+
 	if(GetNumInstruments())
 	{
-		LoadExtendedInstrumentProperties(file, &interpretOpenMPTMade);
+		isOpenMPTMade = LoadExtendedInstrumentProperties(file);
 	}
 
-	LoadExtendedSongProperties(file, &interpretOpenMPTMade);
+	LoadExtendedSongProperties(file, true, &isOpenMPTMade);
 
-	if(interpretOpenMPTMade && m_dwLastSavedWithVersion < MAKE_VERSION_NUMERIC(1, 17, 00, 00))
+	if(isOpenMPTMade && m_dwLastSavedWithVersion < MAKE_VERSION_NUMERIC(1, 17, 00, 00))
 	{
 		// Up to OpenMPT 1.17.02.45 (r165), it was possible that the "last saved with" field was 0
 		// when saving a file in OpenMPT for the first time.
@@ -723,7 +720,7 @@ bool CSoundFile::ReadXM(FileReader &file, ModLoadingFlags loadFlags)
 
 	if(m_dwLastSavedWithVersion >= MAKE_VERSION_NUMERIC(1, 17, 00, 00))
 	{
-		m_madeWithTracker = MPT_USTRING("OpenMPT ") + MptVersion::ToUString(m_dwLastSavedWithVersion);
+		madeWithTracker = U_("OpenMPT ") + m_dwLastSavedWithVersion.ToUString();
 	}
 
 	// We no longer allow any --- or +++ items in the order list now.
@@ -735,6 +732,11 @@ bool CSoundFile::ReadXM(FileReader &file, ModLoadingFlags loadFlags)
 			Order().Replace(0xFF, Order.GetInvalidPatIndex());
 	}
 
+	m_modFormat.formatName = mpt::format(U_("FastTracker 2 v%1.%2"))(fileHeader.version >> 8, mpt::ufmt::hex0<2>(fileHeader.version & 0xFF));
+	m_modFormat.type = U_("xm");
+	m_modFormat.madeWithTracker = std::move(madeWithTracker);
+	m_modFormat.charset = m_dwLastSavedWithVersion ? mpt::CharsetWindows1252 : mpt::CharsetCP437;
+
 	return true;
 }
 
@@ -745,17 +747,8 @@ bool CSoundFile::ReadXM(FileReader &file, ModLoadingFlags loadFlags)
 #define str_pattern				("pattern")
 
 
-bool CSoundFile::SaveXM(const mpt::PathString &filename, bool compatibilityExport)
+bool CSoundFile::SaveXM(std::ostream &f, bool compatibilityExport)
 {
-	if(filename.empty())
-	{
-		return false;
-	}
-	FILE *f = mpt_fopen(filename, "wb");
-	if(!f)
-	{
-		return false;
-	}
 
 	bool addChannel = false; // avoid odd channel count for FT2 compatibility
 
@@ -765,7 +758,7 @@ bool CSoundFile::SaveXM(const mpt::PathString &filename, bool compatibilityExpor
 	memcpy(fileHeader.signature, "Extended Module: ", 17);
 	mpt::String::Write<mpt::String::spacePadded>(fileHeader.songName, m_songName);
 	fileHeader.eof = 0x1A;
-	const std::string openMptTrackerName = MptVersion::GetOpenMPTVersionStr();
+	const std::string openMptTrackerName = mpt::ToCharset(GetCharsetFile(), Version::Current().GetOpenMPTVersionString());
 	mpt::String::Write<mpt::String::spacePadded>(fileHeader.trackerName, openMptTrackerName);
 
 	// Writing song header
@@ -1125,7 +1118,7 @@ bool CSoundFile::SaveXM(const mpt::PathString &filename, bool compatibilityExpor
 		}
 
 		//Save hacked-on extra info
-		SaveMixPlugins(f);
+		SaveMixPlugins(&f);
 		if(GetNumInstruments())
 		{
 			SaveExtendedInstrumentProperties(writeInstruments, f);
@@ -1133,7 +1126,6 @@ bool CSoundFile::SaveXM(const mpt::PathString &filename, bool compatibilityExpor
 		SaveExtendedSongProperties(f);
 	}
 
-	fclose(f);
 	return true;
 }
 
