@@ -2,14 +2,8 @@
  * MixerLoops.cpp
  * --------------
  * Purpose: Utility inner loops for mixer-related functionality.
- * Notes  : 
- *          x86 ( AMD/INTEL ) based low level based mixing functions:
- *          This file contains critical code. The basic X86 functions are
- *          defined at the bottom of the file. #define's are used to isolate
- *          the different flavours of functionality:
- *          ENABLE_MMX, ENABLE_3DNOW, ENABLE_SSE flags must be set to
- *          to compile the optimized sections of the code. In both cases the 
- *          X86_xxxxxx functions will compile.
+ * Notes  : This file contains performance-critical loops with variants
+ *          optimized for various instruction sets.
  * Authors: Olivier Lapicque
  *          OpenMPT Devs
  * The OpenMPT source code is released under the BSD license. Read LICENSE for more details.
@@ -17,15 +11,16 @@
 
 
 #include "stdafx.h"
-
 #include "MixerLoops.h"
-
-#include "Sndfile.h"
+#include "Snd_defs.h"
+#include "ModChannel.h"
+#ifdef ENABLE_SSE2
+#include <emmintrin.h>
+#endif
 
 
 OPENMPT_NAMESPACE_BEGIN
 
-
 ////////////////////////////////////////////////////////////////////////////////////
 // 3DNow! optimizations
 
@@ -162,8 +157,6 @@ mainloop:
 
 #ifdef ENABLE_SSE2
 
-#include <emmintrin.h>
-
 static void SSE2_StereoMixToFloat(const int32 *pSrc, float *pOut1, float *pOut2, uint32 nCount, const float _i2fc)
 {
 	__m128 i2fc = _mm_load_ps1(&_i2fc);
@@ -218,7 +211,7 @@ static void SSE2_FloatToStereoMix(const float *pIn1, const float *pIn2, int32 *p
 #endif // ENABLE_SSE2
 
 
-#ifdef ENABLE_SSE
+#if defined(ENABLE_X86) && defined(ENABLE_SSE)
 
 static void SSE_StereoMixToFloat(const int32 *pSrc, float *pOut1, float *pOut2, uint32 nCount, const float _i2fc)
 {
@@ -275,7 +268,7 @@ mainloop:
 	}
 }
 
-#endif // ENABLE_SSE
+#endif // ENABLE_X86 && ENABLE_SSE
 
 
 
@@ -429,13 +422,13 @@ void StereoMixToFloat(const int32 *pSrc, float *pOut1, float *pOut2, uint32 nCou
 		return;
 	}
 	#endif // ENABLE_SSE2
-	#ifdef ENABLE_SSE
+	#if defined(ENABLE_X86) && defined(ENABLE_SSE)
 		if(GetProcSupport() & PROCSUPPORT_SSE)
 		{
 			SSE_StereoMixToFloat(pSrc, pOut1, pOut2, nCount, _i2fc);
 			return;
 		}
-	#endif // ENABLE_SSE
+	#endif // ENABLE_X86 && ENABLE_SSE
 	#ifdef ENABLE_X86_AMD
 		if(GetProcSupport() & PROCSUPPORT_AMD_3DNOW)
 		{
@@ -482,13 +475,13 @@ void FloatToStereoMix(const float *pIn1, const float *pIn2, int32 *pOut, uint32
 void MonoMixToFloat(const int32 *pSrc, float *pOut, uint32 nCount, const float _i2fc)
 {
 
-	#ifdef ENABLE_SSE
+	#if defined(ENABLE_X86) && defined(ENABLE_SSE)
 		if(GetProcSupport() & PROCSUPPORT_SSE)
 		{
 			SSE_MonoMixToFloat(pSrc, pOut, nCount, _i2fc);
 			return;
 		}
-	#endif // ENABLE_SSE
+	#endif // ENABLE_X86 && ENABLE_SSE
 	#ifdef ENABLE_X86_AMD
 		if(GetProcSupport() & PROCSUPPORT_AMD_3DNOW)
 		{
@@ -506,7 +499,7 @@ void MonoMixToFloat(const int32 *pSrc, float *pOut, uint32 nCount, const float _
 }
 
 
-void FloatToMonoMix(const float *pIn, int *pOut, uint32 nCount, const float _f2ic)
+void FloatToMonoMix(const float *pIn, int32 *pOut, uint32 nCount, const float _f2ic)
 {
 
 	#ifdef ENABLE_X86_AMD
@@ -733,8 +726,8 @@ static void C_StereoFill(mixsample_t *pBuffer, uint32 nSamples, mixsample_t &rof
 #endif
 		rofs -= x_r;
 		lofs -= x_l;
-		pBuffer[i*2] = x_r;
-		pBuffer[i*2+1] = x_l;
+		pBuffer[i*2] = rofs;
+		pBuffer[i*2+1] = lofs;
 	}
 
 #ifndef MPT_INTMIXER
@@ -821,8 +814,8 @@ static void C_EndChannelOfs(ModChannel &chn, mixsample_t *pBuffer, uint32 nSampl
 #endif
 		rofs -= x_r;
 		lofs -= x_l;
-		pBuffer[i*2] += x_r;
-		pBuffer[i*2+1] += x_l;
+		pBuffer[i*2] += rofs;
+		pBuffer[i*2+1] += lofs;
 	}
 #ifndef MPT_INTMIXER
 	if(mpt::abs(rofs) < OFSTHRESHOLD) rofs = 0;
