@@ -58,6 +58,11 @@ void InstrumentEnvelope::Convert(MODTYPE fromType, MODTYPE toType)
 			}
 		}
 	}
+
+	if(toType != MOD_TYPE_MPT)
+	{
+		nReleaseNode = ENV_RELEASE_NODE_UNSET;
+	}
 }
 
 
@@ -105,7 +110,7 @@ int32 InstrumentEnvelope::GetValueFromPosition(int position, int32 rangeOut, int
 		}
 	}
 
-	Limit(value, 0, ENV_PRECISION);
+	Limit(value, int32(0), ENV_PRECISION);
 	return (value * rangeOut + ENV_PRECISION / 2) / ENV_PRECISION;
 }
 
@@ -158,13 +163,13 @@ ModInstrument::ModInstrument(SAMPLEINDEX sample)
 
 	nMixPlug = 0;
 	nVolRampUp = 0;
-	nResampling = SRCMODE_DEFAULT;
+	resampling = SRCMODE_DEFAULT;
 	nCutSwing = 0;
 	nResSwing = 0;
 	nFilterMode = FLTMODE_UNCHANGED;
 	pitchToTempoLock.Set(0);
-	nPluginVelocityHandling = PLUGIN_VELOCITYHANDLING_CHANNEL;
-	nPluginVolumeHandling = PLUGIN_VOLUMEHANDLING_IGNORE;
+	pluginVelocityHandling = PLUGIN_VELOCITYHANDLING_CHANNEL;
+	pluginVolumeHandling = PLUGIN_VOLUMEHANDLING_IGNORE;
 
 	pTuning = CSoundFile::GetDefaultTuning();
 
@@ -313,7 +318,7 @@ void ModInstrument::Sanitize(MODTYPE modType)
 	MPT_UNREFERENCED_PARAMETER(modType);
 	const uint8 range = ENVELOPE_MAX;
 #else
-	const uint8 range = modType == MOD_TYPE_AMS2 ? uint8_max : ENVELOPE_MAX;
+	const uint8 range = modType == MOD_TYPE_AMS ? uint8_max : ENVELOPE_MAX;
 #endif
 	VolEnv.Sanitize();
 	PanEnv.Sanitize();
