@@ -15,9 +15,9 @@
 OPENMPT_NAMESPACE_BEGIN
 
 
-FileTags::FileTags()
+void FileTags::SetEncoder()
 {
-	encoder = mpt::ToUnicode(mpt::CharsetASCII, MptVersion::GetOpenMPTVersionStr());
+	encoder = Version::Current().GetOpenMPTVersionString();
 }
 
 
@@ -29,7 +29,7 @@ mpt::ustring GetSampleNameFromTags(const FileTags &tags)
 		result = tags.title;
 	} else
 	{
-		result = mpt::format(MPT_USTRING("%1 (by %2)"))(tags.title, tags.artist);
+		result = mpt::format(U_("%1 (by %2)"))(tags.title, tags.artist);
 	}
 	return result;
 }
