@@ -10,7 +10,7 @@
 
 #include "stdafx.h"
 #include "Sndfile.h"
-#include "../common/StringFixer.h"
+#include "../common/mptStringBuffer.h"
 #include "../common/version.h"
 
 
@@ -21,7 +21,6 @@ struct UpgradePatternData
 {
 	UpgradePatternData(CSoundFile &sf)
 		: sndFile(sf)
-		, chn(0)
 		, compatPlay(sf.m_playBehaviour[MSF_COMPATIBLE_PLAY]) { }
 
 	void operator() (ModCommand &m)
@@ -222,7 +221,7 @@ struct UpgradePatternData
 	}
 
 	const CSoundFile &sndFile;
-	CHANNELINDEX chn;
+	CHANNELINDEX chn = 0;
 	const bool compatPlay;
 };
 
@@ -336,14 +335,14 @@ void CSoundFile::UpgradeModule()
 	if(m_dwLastSavedWithVersion < MAKE_VERSION_NUMERIC(1, 22, 07, 01))
 	{
 		// Convert ANSI plugin path names to UTF-8 (irrelevant in probably 99% of all cases anyway, I think I've never seen a VST plugin with a non-ASCII file name)
-		for(PLUGINDEX i = 0; i < MAX_MIXPLUGINS; i++)
+		for(auto &plugin : m_MixPlugins)
 		{
 #if defined(MODPLUG_TRACKER)
-			const std::string name = mpt::ToCharset(mpt::CharsetUTF8, mpt::CharsetLocale, m_MixPlugins[i].Info.szLibraryName);
+			const std::string name = mpt::ToCharset(mpt::CharsetUTF8, mpt::CharsetLocale, plugin.Info.szLibraryName);
 #else
-			const std::string name = mpt::ToCharset(mpt::CharsetUTF8, mpt::CharsetWindows1252, m_MixPlugins[i].Info.szLibraryName);
+			const std::string name = mpt::ToCharset(mpt::CharsetUTF8, mpt::CharsetWindows1252, plugin.Info.szLibraryName);
 #endif
-			mpt::String::Copy(m_MixPlugins[i].Info.szLibraryName, name);
+			mpt::String::Copy(plugin.Info.szLibraryName, name);
 		}
 	}
 #endif // NO_PLUGINS
@@ -369,9 +368,9 @@ void CSoundFile::UpgradeModule()
 			if(Instruments[i] != nullptr && Instruments[i]->nVolSwing != 0 && Instruments[i]->nMidiChannel != MidiNoChannel)
 			{
 				bool hasSample = false;
-				for(size_t k = 0; k < CountOf(Instruments[k]->Keyboard); k++)
+				for(auto smp : Instruments[i]->Keyboard)
 				{
-					if(Instruments[i]->Keyboard[k] != 0)
+					if(smp != 0)
 					{
 						hasSample = true;
 						break;
@@ -402,6 +401,30 @@ void CSoundFile::UpgradeModule()
 		}
 	}
 
+	if(m_dwLastSavedWithVersion < MAKE_VERSION_NUMERIC(1, 28, 00, 12))
+	{
+		for(INSTRUMENTINDEX i = 1; i <= GetNumInstruments(); i++) if(Instruments[i] != nullptr)
+		{
+			if(Instruments[i]->VolEnv.nReleaseNode != ENV_RELEASE_NODE_UNSET)
+			{
+				m_playBehaviour.set(kLegacyReleaseNode);
+				break;
+			}
+		}
+	}
+
+	if(m_dwLastSavedWithVersion < MAKE_VERSION_NUMERIC(1, 28, 03, 04))
+	{
+		for(INSTRUMENTINDEX i = 1; i <= GetNumInstruments(); i++) if (Instruments[i] != nullptr)
+		{
+			if(Instruments[i]->pluginVolumeHandling == PLUGIN_VOLUMEHANDLING_MIDI || Instruments[i]->pluginVolumeHandling == PLUGIN_VOLUMEHANDLING_DRYWET)
+			{
+				m_playBehaviour.set(kMIDIVolumeOnNoteOffBug);
+				break;
+			}
+		}
+	}
+
 	Patterns.ForEachModCommand(UpgradePatternData(*this));
 
 	// Convert compatibility flags
@@ -411,7 +434,7 @@ void CSoundFile::UpgradeModule()
 	struct PlayBehaviourVersion
 	{
 		PlayBehaviour behaviour;
-		MptVersion::VersionNum version;
+		Version version;
 	};
 	
 	if(compatModeIT && m_dwLastSavedWithVersion < MAKE_VERSION_NUMERIC(1, 26, 00, 00))
@@ -419,90 +442,90 @@ void CSoundFile::UpgradeModule()
 		// Pre-1.26: Detailed compatibility flags did not exist.
 		static constexpr PlayBehaviourVersion behaviours[] =
 		{
-			{ kTempoClamp,						MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
-			{ kPerChannelGlobalVolSlide,		MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
-			{ kPanOverride,						MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
-			{ kITInstrWithoutNote,				MAKE_VERSION_NUMERIC(1, 17, 02, 46) },
-			{ kITVolColFinePortamento,			MAKE_VERSION_NUMERIC(1, 17, 02, 49) },
-			{ kITArpeggio,						MAKE_VERSION_NUMERIC(1, 17, 02, 49) },
-			{ kITOutOfRangeDelay,				MAKE_VERSION_NUMERIC(1, 17, 02, 49) },
-			{ kITPortaMemoryShare,				MAKE_VERSION_NUMERIC(1, 17, 02, 49) },
-			{ kITPatternLoopTargetReset,		MAKE_VERSION_NUMERIC(1, 17, 02, 49) },
-			{ kITFT2PatternLoop,				MAKE_VERSION_NUMERIC(1, 17, 02, 49) },
-			{ kITPingPongNoReset,				MAKE_VERSION_NUMERIC(1, 17, 02, 51) },
-			{ kITEnvelopeReset,					MAKE_VERSION_NUMERIC(1, 17, 02, 51) },
-			{ kITClearOldNoteAfterCut,			MAKE_VERSION_NUMERIC(1, 17, 02, 52) },
-			{ kITVibratoTremoloPanbrello,		MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
-			{ kITTremor,						MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
-			{ kITRetrigger,						MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
-			{ kITMultiSampleBehaviour,			MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
-			{ kITPortaTargetReached,			MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
-			{ kITPatternLoopBreak,				MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
-			{ kITOffset,						MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
-			{ kITSwingBehaviour,				MAKE_VERSION_NUMERIC(1, 18, 00, 00) },
-			{ kITNNAReset,						MAKE_VERSION_NUMERIC(1, 18, 00, 00) },
-			{ kITSCxStopsSample,				MAKE_VERSION_NUMERIC(1, 18, 00, 01) },
-			{ kITEnvelopePositionHandling,		MAKE_VERSION_NUMERIC(1, 18, 01, 00) },
-			{ kITPortamentoInstrument,			MAKE_VERSION_NUMERIC(1, 19, 00, 01) },
-			{ kITPingPongMode,					MAKE_VERSION_NUMERIC(1, 19, 00, 21) },
-			{ kITRealNoteMapping,				MAKE_VERSION_NUMERIC(1, 19, 00, 30) },
-			{ kITHighOffsetNoRetrig,			MAKE_VERSION_NUMERIC(1, 20, 00, 14) },
-			{ kITFilterBehaviour,				MAKE_VERSION_NUMERIC(1, 20, 00, 35) },
-			{ kITNoSurroundPan,					MAKE_VERSION_NUMERIC(1, 20, 00, 53) },
-			{ kITShortSampleRetrig,				MAKE_VERSION_NUMERIC(1, 20, 00, 54) },
-			{ kITPortaNoNote,					MAKE_VERSION_NUMERIC(1, 20, 00, 56) },
-			{ kRowDelayWithNoteDelay,			MAKE_VERSION_NUMERIC(1, 20, 00, 76) },
-			{ kITDontResetNoteOffOnPorta,		MAKE_VERSION_NUMERIC(1, 20, 02, 06) },
-			{ kITVolColMemory,					MAKE_VERSION_NUMERIC(1, 21, 01, 16) },
-			{ kITPortamentoSwapResetsPos,		MAKE_VERSION_NUMERIC(1, 21, 01, 25) },
-			{ kITEmptyNoteMapSlot,				MAKE_VERSION_NUMERIC(1, 21, 01, 25) },
-			{ kITFirstTickHandling,				MAKE_VERSION_NUMERIC(1, 22, 07, 09) },
-			{ kITSampleAndHoldPanbrello,		MAKE_VERSION_NUMERIC(1, 22, 07, 19) },
-			{ kITClearPortaTarget,				MAKE_VERSION_NUMERIC(1, 23, 04, 03) },
-			{ kITPanbrelloHold,					MAKE_VERSION_NUMERIC(1, 24, 01, 06) },
-			{ kITPanningReset,					MAKE_VERSION_NUMERIC(1, 24, 01, 06) },
-			{ kITPatternLoopWithJumps,			MAKE_VERSION_NUMERIC(1, 25, 00, 19) },
+			{ kTempoClamp,                 MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
+			{ kPerChannelGlobalVolSlide,   MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
+			{ kPanOverride,                MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
+			{ kITInstrWithoutNote,         MAKE_VERSION_NUMERIC(1, 17, 02, 46) },
+			{ kITVolColFinePortamento,     MAKE_VERSION_NUMERIC(1, 17, 02, 49) },
+			{ kITArpeggio,                 MAKE_VERSION_NUMERIC(1, 17, 02, 49) },
+			{ kITOutOfRangeDelay,          MAKE_VERSION_NUMERIC(1, 17, 02, 49) },
+			{ kITPortaMemoryShare,         MAKE_VERSION_NUMERIC(1, 17, 02, 49) },
+			{ kITPatternLoopTargetReset,   MAKE_VERSION_NUMERIC(1, 17, 02, 49) },
+			{ kITFT2PatternLoop,           MAKE_VERSION_NUMERIC(1, 17, 02, 49) },
+			{ kITPingPongNoReset,          MAKE_VERSION_NUMERIC(1, 17, 02, 51) },
+			{ kITEnvelopeReset,            MAKE_VERSION_NUMERIC(1, 17, 02, 51) },
+			{ kITClearOldNoteAfterCut,     MAKE_VERSION_NUMERIC(1, 17, 02, 52) },
+			{ kITVibratoTremoloPanbrello,  MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
+			{ kITTremor,                   MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
+			{ kITRetrigger,                MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
+			{ kITMultiSampleBehaviour,     MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
+			{ kITPortaTargetReached,       MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
+			{ kITPatternLoopBreak,         MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
+			{ kITOffset,                   MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
+			{ kITSwingBehaviour,           MAKE_VERSION_NUMERIC(1, 18, 00, 00) },
+			{ kITNNAReset,                 MAKE_VERSION_NUMERIC(1, 18, 00, 00) },
+			{ kITSCxStopsSample,           MAKE_VERSION_NUMERIC(1, 18, 00, 01) },
+			{ kITEnvelopePositionHandling, MAKE_VERSION_NUMERIC(1, 18, 01, 00) },
+			{ kITPortamentoInstrument,     MAKE_VERSION_NUMERIC(1, 19, 00, 01) },
+			{ kITPingPongMode,             MAKE_VERSION_NUMERIC(1, 19, 00, 21) },
+			{ kITRealNoteMapping,          MAKE_VERSION_NUMERIC(1, 19, 00, 30) },
+			{ kITHighOffsetNoRetrig,       MAKE_VERSION_NUMERIC(1, 20, 00, 14) },
+			{ kITFilterBehaviour,          MAKE_VERSION_NUMERIC(1, 20, 00, 35) },
+			{ kITNoSurroundPan,            MAKE_VERSION_NUMERIC(1, 20, 00, 53) },
+			{ kITShortSampleRetrig,        MAKE_VERSION_NUMERIC(1, 20, 00, 54) },
+			{ kITPortaNoNote,              MAKE_VERSION_NUMERIC(1, 20, 00, 56) },
+			{ kRowDelayWithNoteDelay,      MAKE_VERSION_NUMERIC(1, 20, 00, 76) },
+			{ kITDontResetNoteOffOnPorta,  MAKE_VERSION_NUMERIC(1, 20, 02, 06) },
+			{ kITVolColMemory,             MAKE_VERSION_NUMERIC(1, 21, 01, 16) },
+			{ kITPortamentoSwapResetsPos,  MAKE_VERSION_NUMERIC(1, 21, 01, 25) },
+			{ kITEmptyNoteMapSlot,         MAKE_VERSION_NUMERIC(1, 21, 01, 25) },
+			{ kITFirstTickHandling,        MAKE_VERSION_NUMERIC(1, 22, 07, 09) },
+			{ kITSampleAndHoldPanbrello,   MAKE_VERSION_NUMERIC(1, 22, 07, 19) },
+			{ kITClearPortaTarget,         MAKE_VERSION_NUMERIC(1, 23, 04, 03) },
+			{ kITPanbrelloHold,            MAKE_VERSION_NUMERIC(1, 24, 01, 06) },
+			{ kITPanningReset,             MAKE_VERSION_NUMERIC(1, 24, 01, 06) },
+			{ kITPatternLoopWithJumps,     MAKE_VERSION_NUMERIC(1, 25, 00, 19) },
 		};
 
 		for(const auto &b : behaviours)
 		{
-			m_playBehaviour.set(b.behaviour, (m_dwLastSavedWithVersion >= b.version || m_dwLastSavedWithVersion == (b.version & 0xFFFF0000)));
+			m_playBehaviour.set(b.behaviour, (m_dwLastSavedWithVersion >= b.version || m_dwLastSavedWithVersion == b.version.Masked(0xFFFF0000u)));
 		}
 	} else if(compatModeXM && m_dwLastSavedWithVersion < MAKE_VERSION_NUMERIC(1, 26, 00, 00))
 	{
 		// Pre-1.26: Detailed compatibility flags did not exist.
 		static constexpr PlayBehaviourVersion behaviours[] =
 		{
-			{ kTempoClamp,						MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
-			{ kPerChannelGlobalVolSlide,		MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
-			{ kPanOverride,						MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
-			{ kITFT2PatternLoop,				MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
-			{ kFT2Arpeggio,						MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
-			{ kFT2Retrigger,					MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
-			{ kFT2VolColVibrato,				MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
-			{ kFT2PortaNoNote,					MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
-			{ kFT2KeyOff,						MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
-			{ kFT2PanSlide,						MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
-			{ kFT2OffsetOutOfRange,				MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
-			{ kFT2RestrictXCommand,				MAKE_VERSION_NUMERIC(1, 18, 00, 00) },
-			{ kFT2RetrigWithNoteDelay,			MAKE_VERSION_NUMERIC(1, 18, 00, 00) },
-			{ kFT2SetPanEnvPos,					MAKE_VERSION_NUMERIC(1, 18, 00, 00) },
-			{ kFT2PortaIgnoreInstr,				MAKE_VERSION_NUMERIC(1, 18, 00, 01) },
-			{ kFT2VolColMemory,					MAKE_VERSION_NUMERIC(1, 18, 01, 00) },
-			{ kFT2LoopE60Restart,				MAKE_VERSION_NUMERIC(1, 18, 02, 01) },
-			{ kFT2ProcessSilentChannels,		MAKE_VERSION_NUMERIC(1, 18, 02, 01) },
-			{ kFT2ReloadSampleSettings,			MAKE_VERSION_NUMERIC(1, 20, 00, 36) },
-			{ kFT2PortaDelay,					MAKE_VERSION_NUMERIC(1, 20, 00, 40) },
-			{ kFT2Transpose,					MAKE_VERSION_NUMERIC(1, 20, 00, 62) },
-			{ kFT2PatternLoopWithJumps,			MAKE_VERSION_NUMERIC(1, 20, 00, 69) },
-			{ kFT2PortaTargetNoReset,			MAKE_VERSION_NUMERIC(1, 20, 00, 69) },
-			{ kFT2EnvelopeEscape,				MAKE_VERSION_NUMERIC(1, 20, 00, 77) },
-			{ kFT2Tremor,						MAKE_VERSION_NUMERIC(1, 20, 01, 11) },
-			{ kFT2OutOfRangeDelay,				MAKE_VERSION_NUMERIC(1, 20, 02, 02) },
-			{ kFT2Periods,						MAKE_VERSION_NUMERIC(1, 22, 03, 01) },
-			{ kFT2PanWithDelayedNoteOff,		MAKE_VERSION_NUMERIC(1, 22, 03, 02) },
-			{ kFT2VolColDelay,					MAKE_VERSION_NUMERIC(1, 22, 07, 19) },
-			{ kFT2FinetunePrecision,			MAKE_VERSION_NUMERIC(1, 22, 07, 19) },
+			{ kTempoClamp,               MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
+			{ kPerChannelGlobalVolSlide, MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
+			{ kPanOverride,              MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
+			{ kITFT2PatternLoop,         MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
+			{ kFT2Arpeggio,              MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
+			{ kFT2Retrigger,             MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
+			{ kFT2VolColVibrato,         MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
+			{ kFT2PortaNoNote,           MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
+			{ kFT2KeyOff,                MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
+			{ kFT2PanSlide,              MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
+			{ kFT2OffsetOutOfRange,      MAKE_VERSION_NUMERIC(1, 17, 03, 02) },
+			{ kFT2RestrictXCommand,      MAKE_VERSION_NUMERIC(1, 18, 00, 00) },
+			{ kFT2RetrigWithNoteDelay,   MAKE_VERSION_NUMERIC(1, 18, 00, 00) },
+			{ kFT2SetPanEnvPos,          MAKE_VERSION_NUMERIC(1, 18, 00, 00) },
+			{ kFT2PortaIgnoreInstr,      MAKE_VERSION_NUMERIC(1, 18, 00, 01) },
+			{ kFT2VolColMemory,          MAKE_VERSION_NUMERIC(1, 18, 01, 00) },
+			{ kFT2LoopE60Restart,        MAKE_VERSION_NUMERIC(1, 18, 02, 01) },
+			{ kFT2ProcessSilentChannels, MAKE_VERSION_NUMERIC(1, 18, 02, 01) },
+			{ kFT2ReloadSampleSettings,  MAKE_VERSION_NUMERIC(1, 20, 00, 36) },
+			{ kFT2PortaDelay,            MAKE_VERSION_NUMERIC(1, 20, 00, 40) },
+			{ kFT2Transpose,             MAKE_VERSION_NUMERIC(1, 20, 00, 62) },
+			{ kFT2PatternLoopWithJumps,  MAKE_VERSION_NUMERIC(1, 20, 00, 69) },
+			{ kFT2PortaTargetNoReset,    MAKE_VERSION_NUMERIC(1, 20, 00, 69) },
+			{ kFT2EnvelopeEscape,        MAKE_VERSION_NUMERIC(1, 20, 00, 77) },
+			{ kFT2Tremor,                MAKE_VERSION_NUMERIC(1, 20, 01, 11) },
+			{ kFT2OutOfRangeDelay,       MAKE_VERSION_NUMERIC(1, 20, 02, 02) },
+			{ kFT2Periods,               MAKE_VERSION_NUMERIC(1, 22, 03, 01) },
+			{ kFT2PanWithDelayedNoteOff, MAKE_VERSION_NUMERIC(1, 22, 03, 02) },
+			{ kFT2VolColDelay,           MAKE_VERSION_NUMERIC(1, 22, 07, 19) },
+			{ kFT2FinetunePrecision,     MAKE_VERSION_NUMERIC(1, 22, 07, 19) },
 		};
 
 		for(const auto &b : behaviours)
@@ -516,16 +539,17 @@ void CSoundFile::UpgradeModule()
 		// The following behaviours were added in/after OpenMPT 1.26, so are not affected by the upgrade mechanism above.
 		static constexpr PlayBehaviourVersion behaviours[] =
 		{
-			{ kITInstrWithNoteOff,				MAKE_VERSION_NUMERIC(1, 26, 00, 01) },
-			{ kITMultiSampleInstrumentNumber,	MAKE_VERSION_NUMERIC(1, 27, 00, 27) },
+			{ kITInstrWithNoteOff,            MAKE_VERSION_NUMERIC(1, 26, 00, 01) },
+			{ kITMultiSampleInstrumentNumber, MAKE_VERSION_NUMERIC(1, 27, 00, 27) },
+			{ kITInstrWithNoteOffOldEffects,  MAKE_VERSION_NUMERIC(1, 28, 02, 06) },
 		};
 
 		for(const auto &b : behaviours)
 		{
-			if(m_dwLastSavedWithVersion < (b.version & 0xFFFF0000))
+			if(m_dwLastSavedWithVersion < b.version.Masked(0xFFFF0000u))
 				m_playBehaviour.reset(b.behaviour);
 			// Full version information available, i.e. not compatibility-exported.
-			else if(m_dwLastSavedWithVersion > (b.version & 0xFFFF0000) && m_dwLastSavedWithVersion < b.version)
+			else if(m_dwLastSavedWithVersion > b.version.Masked(0xFFFF0000u) && m_dwLastSavedWithVersion < b.version)
 				m_playBehaviour.reset(b.behaviour);
 		}
 	} else if(GetType() == MOD_TYPE_XM)
@@ -533,10 +557,12 @@ void CSoundFile::UpgradeModule()
 		// The following behaviours were added after OpenMPT 1.26, so are not affected by the upgrade mechanism above.
 		static constexpr PlayBehaviourVersion behaviours[] =
 		{
-			{ kFT2NoteOffFlags,					MAKE_VERSION_NUMERIC(1, 27, 00, 27) },
-			{ kRowDelayWithNoteDelay,			MAKE_VERSION_NUMERIC(1, 27, 00, 37) },
-			{ kFT2TremoloRampWaveform,			MAKE_VERSION_NUMERIC(1, 27, 00, 37) },
-			{ kFT2PortaUpDownMemory,			MAKE_VERSION_NUMERIC(1, 27, 00, 37) },
+			{ kFT2NoteOffFlags,          MAKE_VERSION_NUMERIC(1, 27, 00, 27) },
+			{ kRowDelayWithNoteDelay,    MAKE_VERSION_NUMERIC(1, 27, 00, 37) },
+			{ kFT2TremoloRampWaveform,   MAKE_VERSION_NUMERIC(1, 27, 00, 37) },
+			{ kFT2PortaUpDownMemory,     MAKE_VERSION_NUMERIC(1, 27, 00, 37) },
+			{ kFT2PanSustainRelease,     MAKE_VERSION_NUMERIC(1, 28, 00, 09) },
+			{ kFT2NoteDelayWithoutInstr, MAKE_VERSION_NUMERIC(1, 28, 00, 44) },
 		};
 
 		for(const auto &b : behaviours)
@@ -549,13 +575,14 @@ void CSoundFile::UpgradeModule()
 		// We do not store any of these flags in S3M files.
 		static constexpr PlayBehaviourVersion behaviours[] =
 		{
-			{ kST3NoMutedChannels,		MAKE_VERSION_NUMERIC(1, 18, 00, 00) },
-			{ kST3EffectMemory,			MAKE_VERSION_NUMERIC(1, 20, 00, 00) },
-			{ kRowDelayWithNoteDelay,	MAKE_VERSION_NUMERIC(1, 20, 00, 00) },
-			{ kST3PortaSampleChange,	MAKE_VERSION_NUMERIC(1, 22, 00, 00) },
-			{ kST3VibratoMemory,		MAKE_VERSION_NUMERIC(1, 26, 00, 00) },
-			{ kITPanbrelloHold,			MAKE_VERSION_NUMERIC(1, 26, 00, 00) },
-			{ KST3PortaAfterArpeggio,	MAKE_VERSION_NUMERIC(1, 27, 00, 00) },
+			{ kST3NoMutedChannels,         MAKE_VERSION_NUMERIC(1, 18, 00, 00) },
+			{ kST3EffectMemory,            MAKE_VERSION_NUMERIC(1, 20, 00, 00) },
+			{ kRowDelayWithNoteDelay,      MAKE_VERSION_NUMERIC(1, 20, 00, 00) },
+			{ kST3PortaSampleChange,       MAKE_VERSION_NUMERIC(1, 22, 00, 00) },
+			{ kST3VibratoMemory,           MAKE_VERSION_NUMERIC(1, 26, 00, 00) },
+			{ kITPanbrelloHold,            MAKE_VERSION_NUMERIC(1, 26, 00, 00) },
+			{ KST3PortaAfterArpeggio,      MAKE_VERSION_NUMERIC(1, 27, 00, 00) },
+			{ kST3OffsetWithoutInstrument, MAKE_VERSION_NUMERIC(1, 28, 00, 00) },
 		};
 
 		for(const auto &b : behaviours)
@@ -565,6 +592,12 @@ void CSoundFile::UpgradeModule()
 		}
 	}
 
+	if(GetType() == MOD_TYPE_XM && m_dwLastSavedWithVersion < MAKE_VERSION_NUMERIC(1, 19, 00, 00))
+	{
+		// This bug was introduced sometime between 1.18.03.00 and 1.19.01.00
+		m_playBehaviour.set(kFT2NoteDelayWithoutInstr);
+	}
+
 	if(m_dwLastSavedWithVersion >= MAKE_VERSION_NUMERIC(1, 27, 00, 27) && m_dwLastSavedWithVersion < MAKE_VERSION_NUMERIC(1, 27, 00, 49))
 	{
 		// OpenMPT 1.27 inserted some IT/FT2 flags before the S3M flags that are never saved to files anyway, to keep the flag IDs a bit more compact.
@@ -597,7 +630,22 @@ void CSoundFile::UpgradeModule()
 		// Frequency slides were always in Hz rather than periods in this version range.
 		m_playBehaviour.set(kHertzInLinearMode);
 	}
+
+	if(m_playBehaviour[kITEnvelopePositionHandling]
+		&& m_dwLastSavedWithVersion >= MAKE_VERSION_NUMERIC(1, 23, 01, 02) && m_dwLastSavedWithVersion < MAKE_VERSION_NUMERIC(1, 28, 00, 43))
+	{
+		// Bug that effectively clamped the release node to the sustain end
+		for(INSTRUMENTINDEX i = 1; i <= GetNumInstruments(); i++) if(Instruments[i] != nullptr)
+		{
+			if(Instruments[i]->VolEnv.nReleaseNode != ENV_RELEASE_NODE_UNSET
+				&& Instruments[i]->VolEnv.dwFlags[ENV_SUSTAIN]
+				&& Instruments[i]->VolEnv.nReleaseNode > Instruments[i]->VolEnv.nSustainEnd)
+			{
+				m_playBehaviour.set(kReleaseNodePastSustainBug);
+				break;
+			}
+		}
+	}
 }
 
-
 OPENMPT_NAMESPACE_END
